/*
 * Project name:
     MMC
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101112:
       - initial release;
 * Description:
     MMC unit test. Upon flashing, insert a MMC/SD card into the
     module, when you should receive the "Init-OK" message.
     Then, you can experiment with MMC read and write functions,
     and observe the results through the Usart Terminal (Ctrl + T).
 * Test configuration:
     MCU:             dsPIC33FJ128GP710
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70286C.pdf
     Dev.Board:       mikroMMB for dsPIC33 Board
                      http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
     Oscillator:      XT-PLL 4x, 8.00000 MHz
                      ac:MMC module
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - This is NOT an example for working with FILES on a MMC/SD memory module.
     - This example should work both for MMC and SD media types.
 */

// MMC module connections
sbit Mmc_Chip_Select           at LATG9_bit;  // for writing to output pin always use latch
sbit Mmc_Chip_Select_Direction at TRISG9_bit;
// eof MMC module connections

// Variables for MMC routines
unsigned char SectorData[512];        // Buffer for MMC sector reading/writing
unsigned char data_for_registers[16]; // buffer for CID and CSD registers

// UART2 write text and new line (carriage return + line feed)
void UART2_Write_Line(char *uart_text) {
     UART2_Write_Text(uart_text);
     UART2_Write(13);
     UART2_Write(10);
}

// Display byte in hex
void PrintHex(unsigned char i) {
  unsigned char hi,lo;

  hi = i & 0xF0;               // High nibble
  hi = hi >> 4;
  hi = hi + '0';
  if (hi>'9') hi=hi+7;
  lo = (i & 0x0F) + '0';       // Low nibble
  if (lo>'9') lo=lo+7;

  UART2_Write(hi);
  UART2_Write(lo);
}

  const char   FILL_CHAR = 'm';
  unsigned int i, SectorNo;
  char         mmc_error;
  bit          data_ok;

void main() {

  TRISC2_bit = 0;
  LATC2_bit = 0;       // Turn off TFT backlight

  // Setup pll
  PLLPRE_0_bit = 0;
  PLLPRE_1_bit = 0;
  PLLPRE_2_bit = 0;
  PLLPRE_3_bit = 0;

  PLLDIV_0_bit = 0;
  PLLDIV_1_bit = 1;
  PLLDIV_2_bit = 1;
  PLLDIV_3_bit = 1;
  PLLDIV_4_bit = 1;
  PLLDIV_5_bit = 0;
  PLLDIV_6_bit = 0;
  PLLDIV_7_bit = 0;

  PLLPOST_0_bit=0;
  PLLPOST_1_bit=1;
  // Setup pll end

  // Disable other peripheral modules on the same SPI bus
  TRISC4_bit = 0;
  LATC4_bit = 1;              // Disable Serial Flash module
  TRISG12_bit = 0;
  LATG12_bit = 1;             // Disable MP3 module
  
  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;
  UART2_Init(9600);               // Initialize UART module at 9600 bps
  UART_Set_Active(UART2_Read, UART2_Write, UART2_Data_Ready, UART2_Tx_Idle);

  Delay_ms(100);                  // Wait for UART module to stabilize
 
  //--- initialise a MMC card
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_1, _SPI_PRESCALE_PRI_64, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
  UART2_Write_Line("MMC Init-OK");
 
  // initialise a MMC card
  mmc_error = Mmc_Init();
  if(mmc_error == 0)
    UART2_Write_Line("MMC Init-OK");          // If MMC present report
  else
    UART2_Write_Line("MMC Init-error");       // If error report
    
  // Fill MMC buffer with same characters
  for(i=0; i<=511; i++)
    SectorData[i] = FILL_CHAR;

  // Choose sector
  SectorNo = 590;

  // Write sector
  mmc_error = Mmc_Write_Sector(SectorNo, SectorData);
  if(mmc_error == 0)
    UART2_Write_Line("Write-OK");
  else  // if there are errors.....
    UART2_Write_Line("Write-Error");

  // Reading of CID register
  mmc_error = Mmc_Read_Cid(data_for_registers);
  if(mmc_error == 0) {
      UART2_Write_Text("CID : ");
      for(i=0; i<=15; i++)
        PrintHex(data_for_registers[i]);
      UART2_Write_Line("");
  }
  else
    UART2_Write_Line("CID-error");

  // Reading of CSD register
  mmc_error = Mmc_Read_Csd(data_for_registers);
  if(mmc_error == 0) {
    UART2_Write_Text("CSD : ");
    for(i=0; i<=15; i++)
      PrintHex(data_for_registers[i]);
    UART2_Write_Line("");
  }
  else
    UART2_Write_Line("CSD-error");

  // Read sector
  mmc_error = Mmc_Read_Sector(SectorNo, SectorData);
  if(mmc_error == 0) {
    UART2_Write_Line("Read-OK");
    // Chech data match
    data_ok = 1;
    for(i=0; i<=511; i++) {
      UART2_Write(SectorData[i]);
      if (SectorData[i] != FILL_CHAR) {
        data_ok = 0;
        break;
      }
    }
    UART2_Write_Line("");
    if (data_ok)
      UART2_Write_Line("Content-OK");
    else
      UART2_Write_Line("Content-Error");
  }
  else                     // If there are errors.....
    UART2_Write_Line("Read-Error");
                          // Signal test end
    UART2_Write_Line("Test End.");
}