/*
 * Project name:
     MP3
 * Copyright:
     (c) MikroElektronika, 2009.
 * Revision History:
     20101112:
       - initial release
 * Description:
     MP3 unit test. Upon flashing, insert a MMC/SD card into the
     module, when you should receive the "Init-OK" message.
     Then, you can experiment with MMC read and write functions,
     and observe the results through the Usart Terminal (Ctrl + T).
 * Test configuration:
     MCU:             dsPIC33FJ128GP710
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70286C.pdf
     Dev.Board:       mikroMMB for dsPIC33 Board
                      http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
     Oscillator:      XT-PLL 4x, 8.00000 MHz
                      ac:MP3 unit
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - On the SD cart which is FAT formatted, you should have file: sound1.mp3
 */

#include <built_in.h>

// Smart MP3 board connections
sbit Mmc_Chip_Select           at LATG9_bit;
sbit MP3_CS                    at RG12_bit;
sbit MP3_RST                   at RG13_bit;
sbit DREQ                      at RG1_bit;
sbit BSYNC                     at RG14_bit;
sbit DCLK                      at LATG0_bit;
sbit SDATA                     at RA6_bit;

sbit Mmc_Chip_Select_Direction at TRISG9_bit;
sbit MP3_CS_Direction          at TRISG12_bit;
sbit MP3_RST_Direction         at TRISG13_bit;
sbit DREQ_Direction            at TRISG1_bit;
sbit BSYNC_Direction           at TRISG14_bit;
sbit DCLK_Direction            at TRISG0_bit;
sbit SDATA_Direction           at TRISA6_bit;

// VS1011E constants
const char WRITE_CODE  = 0x02;
const char READ_CODE   = 0x03;
const char MODE_ADDR   = 0x00;
const char CLOCKF_ADDR = 0x03;
const char VOL_ADDR    = 0x0B;

// global variables
char          filename[14] = "sound1.mp3";   // MP3 file name
unsigned long i, file_size;
char  data_buffer_32[32];
const BUFFER_SIZE = 2048;
char  BufferLarge[BUFFER_SIZE];
char  volume_left, volume_right;

// Write text and goto new line CR and LF
void UART2_Write_Line (char *text) {
  UART2_Write_Text(text);
  UART2_Write(13);
  UART2_Write(10);
}

// Writes one byte to MP3 SDI
void SW_SPI_Write(char data_) {
  BSYNC = 1;                              // Set BSYNC before sending the first bit

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;                            // bitorder is LSB first

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  BSYNC = 0;                              // Clear BSYNC after sending the second bit

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
  SDATA = data_;
  DCLK = 1;
  data_ >>= 1;

  DCLK = 0;
}

// Writes one word to MP3 SCI
void MP3_SCI_Write(char address, unsigned int data_in) {
  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(WRITE_CODE);
  SPI2_Write(address);
  SPI2_Write(Hi(data_in));                // high byte
  SPI2_Write(Lo(data_in));                // low byte
  MP3_CS = 1;                             // deselect MP3 SCI
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(READ_CODE);
  SPI2_Write(start_address);

  while (words_count--) {                 // read words_count words byte per byte
    temp = SPI2_Read(0);
    temp <<= 8;
    temp += SPI2_Read(0);
    *(data_buffer++) = temp;
  }
  MP3_CS = 1;                             // deselect MP3 SCI
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_) {
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SW_SPI_Write(data_);
}

// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_) {
  char i;

  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
    SW_SPI_Write(data_[i]);
}

// Set clock
void Set_Clock(unsigned int clock_khz, char doubler) {
  clock_khz /= 2;                         // Calculate value
  if (doubler) {
    clock_khz |= 0x8000;
  }
  MP3_SCI_Write(CLOCKF_ADDR, clock_khz);  // Write value to CLOCKF register
}

// Set volume
void Set_Volume(char left,char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // Calculate value
  MP3_SCI_Write(VOL_ADDR, volume);        // Write value to VOL register
}
// If volume Buttons are pressed update volume

// Software Reset
void Soft_Reset(char fill_zeros) {
  char i;

  MP3_SCI_Write(MODE_ADDR,0x0204);        // Write to MODE register
                                          // Set SM_RESET bit and SM_BITORD bit(bitorder is LSB first)
  Delay_us(2);                            // Required, see MP3 codec datasheet -> Software Reset
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  if (fill_zeros) {                       // feed 2048 zeros to the MP3 SDI bus:
    for (i=0; i<32; i++) {                // clear 32byte buffer
      data_buffer_32[i] = 0;
    }
    for (i=0; i < 2048/32; i++) {         // use 32byte buffer to clear MP3 buffer
      MP3_SDI_Write_32(data_buffer_32);
    }
  }
  else
    MP3_SDI_Write(0);                     // write zero once
}
// Initialize VS1011E
void MP3_init() {
  Soft_Reset(0);                          // SW Reset, do not feed zeros

  Set_Clock(25000,0);                     // Set clock to 25MHz, do not use clock doubler

  volume_left  = 10;                      // Set volume to initial value
  volume_right = 10;
  Set_Volume(volume_left,volume_right);
}

void Init() {
  ADPCFG = 0xFFFF;        // set all AN pins to digital

  DCLK_Direction  = 0;                    // Set SW SPI pin directions to output
  SDATA_Direction = 0;
  DCLK  = 0;                              // Clear SW SPI SCK
  SDATA = 0;                              // Clear SW SPI SDO

  MP3_CS_Direction  = 0;                  // Configure MP3_CS as output
  MP3_CS            = 1;                  // Deselect MP3_CS
  MP3_RST_Direction = 0;                  // Configure MP3_RST as output
  MP3_RST           = 1;                  // Set MP3_RST pin

  DREQ_Direction  = 1;                    // Configure DREQ as input
  BSYNC_Direction = 0;                    // Configure BSYNC as output
  BSYNC           = 0;                    // Clear BSYNC
}

void main() {

  TRISC2_bit = 0;
  LATC2_bit = 0;   // Turn off TFT backlight

  // Setup PLL
  PLLPRE_0_bit = 0;
  PLLPRE_1_bit = 0;
  PLLPRE_2_bit = 0;
  PLLPRE_3_bit = 0;

  PLLDIV_0_bit = 0;
  PLLDIV_1_bit = 1;
  PLLDIV_2_bit = 1;
  PLLDIV_3_bit = 1;
  PLLDIV_4_bit = 1;
  PLLDIV_5_bit = 0;
  PLLDIV_6_bit = 0;
  PLLDIV_7_bit = 0;

  PLLPOST_0_bit = 0;
  PLLPOST_1_bit = 1;
  // Setup PLL end

  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;
  
   // Disable other peripheral modules on the same SPI bus
  TRISC4_bit = 0;
  LATC4_bit = 1;   // Disable Serial Flash module
  
  PORTB = 0;
  PORTD = 0;

   Init();

  // Initialize USART for signaling
  UART2_Init(9600);
  Delay_100ms();

  UART2_Write_Line("Initializing SPI");
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_1, _SPI_PRESCALE_PRI_4,
                     _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);

  UART2_Write_Line("Initializing VS1011E decoder interface");
  MP3_init();

  UART2_Write_Line("Initializing MMC_FAT");
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&filename, 0) ) {
      UART2_Write_Line("File Assigned");
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          // Procedure returns size of the file
      // Send file blocks to MP3 SDI
      while (file_size > BUFFER_SIZE) {

        for (i=0; i<BUFFER_SIZE; i++) {
          Mmc_Fat_Read(BufferLarge + i);
        }

        for (i=0; i<BUFFER_SIZE/32; i++) {
          MP3_SDI_Write_32(BufferLarge + i*32);
        }

        file_size -= BUFFER_SIZE;

//        Check_Volume();
      }
      // Send the rest of the file to MP3 SDI
      for (i=0; i<file_size; i++) {
        Mmc_Fat_Read(BufferLarge + i);
      }
      for (i=0; i<file_size; i++) {
        MP3_SDI_Write(BufferLarge[i]);
      }
    }
    else {
      UART2_Write_Line("File not assigned");
    }

  }
  else {
    UART2_Write_Line("MMC FAT not initialized");
  }
}