/*
 * Project name:
     SerialFlash example (Demonstration of usage of the SerialFlash board)
 * Copyright:
     (c) MikroElektronika, 2011.
 * Revision History:
     20101112:
       - initial release;
       - 20110209(SZ) - included TFT display;
 * Description:
      Demonstration of SerialFlash usage (chip erase, read Flash ID, simple writing and reading back a simple byte,
      simple writing and reading back an array of 16 bytes).
 * Test configuration:
     MCU:             dsPIC33FJ128GP710
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70286C.pdf
     Dev.Board:       mikroMMB for dsPIC33 Board - ac:Serial_Flash unit
                      http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
     Oscillator:      XT-PLL, 32.00000 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
        - none.
 */

#include <built_in.h>
#define _DATA_ARRAY_SIZE 16
#include "Serial_Flash_driver.h"

/*********************************************************************/
unsigned short TFT_DataPort at LATD;              // DATA port
unsigned short TFT_DataPort_Direction at TRISD;

sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_RST  at LATC1_bit;

sbit TFT_RS_Direction  at TRISB15_bit;
sbit TFT_RS   at LATB15_bit;

sbit TFT_CS_Direction  at TRISC3_bit;
sbit TFT_CS   at LATC3_bit;

sbit TFT_RD_Direction  at TRISD12_bit;
sbit TFT_RD   at LATD12_bit;

sbit TFT_WR_Direction  at TRISD13_bit;
sbit TFT_WR   at LATD13_bit;


sbit CS_Serial_Flash_bit at LATC4_bit;
sbit CS_Serial_Flash_Direction_bit at TRISC4_bit;

/*********************************************************************/

// write into index register IR, then send command or data
void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}


unsigned char write_array[_DATA_ARRAY_SIZE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
unsigned char read_array[_DATA_ARRAY_SIZE] = {0};

void main() {
  unsigned char temp, SerialFlashID, txt[12];
  unsigned char i, to_write, success;
  unsigned long address;
  unsigned int half_address;

  // Set PLL : Fosc = ((Fin/PLLPRE)*PLLDIV)/PLLPOST ; (((8MHz/2)*32)/4) = 32MHz
  // refer the family datasheet for more details
  CLKDIVbits.PLLPRE = 0;
  PLLFBDbits.PLLDIV = 0x1E;
  CLKDIVbits.PLLPOST = 1;

  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;
  AD2PCFGL = 0xFFFF;

  TRISE = 0;
  TRISD = 0;
  Delay_ms(150);
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(320, 240);

  TFT_Fill_Screen(CL_AQUA);

  TRISG12_bit = 0;
  LATG12_bit = 1;

  //Disable other peripheral modules on the same SPI bus
  TRISG9_bit = 0;
  LATG9_bit = 1;    // Disable MMC
  TRISG12_bit = 0;
  LATG12_bit = 1;   // Disable MP3 module
  //

  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_2, _SPI_PRESCALE_PRI_1, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_END, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);

  SerialFlash_init();
  SerialFlash_WriteEnable();
  Delay_ms(500);

  TFT_Write_Text("***Serial Flash Demo***", 70, 15);

    /************************************************************************
    * Erase entire flash
    ************************************************************************/
      TFT_Write_Text("Erasing chip... ", 110, 45);
      SerialFlash_ChipErase();
      TFT_Write_Text("Done!", 110, 65);

    /************************************************************************
    * Read Flash ID
    ************************************************************************/

      SerialFlashID = SerialFlash_ReadID();
      ByteToStr(SerialFlashID, txt);
      TFT_Write_Text("Flash ID:", 110, 95);
      TFT_Write_Text(txt, 180, 95);
      Delay_ms(500);

    /************************************************************************
    * Write and read back a single byte
    ************************************************************************/


      temp = 221;
      TFT_Write_Text("Writting:", 110, 125);
      ByteToStr(temp, txt);
      TFT_Write_Text(txt, 180, 125);
      SerialFlash_WriteByte(temp, 0x123456);

      TFT_Write_Text("Reading: ", 110, 145);
      temp = SerialFlash_ReadByte(0x123456);
      ByteToStr(temp, txt);
      TFT_Write_Text(txt, 180, 145);
      Delay_ms(500);
    /************************************************************************
    * Write the repetitive pattern of data to fill the first 4K of memory
    * and then read the entire flash again and check the data correctness
    ************************************************************************/
      TFT_Write_Text("Writing array...", 110, 175);
      SerialFlash_WriteArray(0x0000, &write_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      TFT_Write_Text("Reading array...", 110, 195);
      SerialFlash_ReadArray(0x0000, &read_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      success = 1;
      for (i = 0; i<_DATA_ARRAY_SIZE; i++){
        if (read_array[i] != write_array[i]){
          success = 0;
          break;
        }
      }
      if (success)
      TFT_Write_Text("Success - Full match!", 110, 215);
      else
      TFT_Write_Text("Failed", 110, 215);
}